import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import dayjs from 'dayjs/esm';

import { isPresent } from 'app/core/util/operators';
import { ApplicationConfigService } from 'app/core/config/application-config.service';
import { createRequestOption } from 'app/core/request/request-util';
import { IReviewHistoryEntry, getReviewHistoryEntryIdentifier } from '../review-history-entry.model';

export type EntityResponseType = HttpResponse<IReviewHistoryEntry>;
export type EntityArrayResponseType = HttpResponse<IReviewHistoryEntry[]>;

@Injectable({ providedIn: 'root' })
export class ReviewHistoryEntryService {
  protected resourceUrl = this.applicationConfigService.getEndpointFor('api/review-history-entries');

  constructor(protected http: HttpClient, protected applicationConfigService: ApplicationConfigService) {}

  create(reviewHistoryEntry: IReviewHistoryEntry): Observable<EntityResponseType> {
    const copy = this.convertDateFromClient(reviewHistoryEntry);
    return this.http
      .post<IReviewHistoryEntry>(this.resourceUrl, copy, { observe: 'response' })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  update(reviewHistoryEntry: IReviewHistoryEntry): Observable<EntityResponseType> {
    const copy = this.convertDateFromClient(reviewHistoryEntry);
    return this.http
      .put<IReviewHistoryEntry>(`${this.resourceUrl}/${getReviewHistoryEntryIdentifier(reviewHistoryEntry) as number}`, copy, {
        observe: 'response',
      })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  partialUpdate(reviewHistoryEntry: IReviewHistoryEntry): Observable<EntityResponseType> {
    const copy = this.convertDateFromClient(reviewHistoryEntry);
    return this.http
      .patch<IReviewHistoryEntry>(`${this.resourceUrl}/${getReviewHistoryEntryIdentifier(reviewHistoryEntry) as number}`, copy, {
        observe: 'response',
      })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  find(id: number): Observable<EntityResponseType> {
    return this.http
      .get<IReviewHistoryEntry>(`${this.resourceUrl}/${id}`, { observe: 'response' })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  query(req?: any): Observable<EntityArrayResponseType> {
    const options = createRequestOption(req);
    return this.http
      .get<IReviewHistoryEntry[]>(this.resourceUrl, { params: options, observe: 'response' })
      .pipe(map((res: EntityArrayResponseType) => this.convertDateArrayFromServer(res)));
  }

  delete(id: number): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  addReviewHistoryEntryToCollectionIfMissing(
    reviewHistoryEntryCollection: IReviewHistoryEntry[],
    ...reviewHistoryEntriesToCheck: (IReviewHistoryEntry | null | undefined)[]
  ): IReviewHistoryEntry[] {
    const reviewHistoryEntries: IReviewHistoryEntry[] = reviewHistoryEntriesToCheck.filter(isPresent);
    if (reviewHistoryEntries.length > 0) {
      const reviewHistoryEntryCollectionIdentifiers = reviewHistoryEntryCollection.map(
        reviewHistoryEntryItem => getReviewHistoryEntryIdentifier(reviewHistoryEntryItem)!
      );
      const reviewHistoryEntriesToAdd = reviewHistoryEntries.filter(reviewHistoryEntryItem => {
        const reviewHistoryEntryIdentifier = getReviewHistoryEntryIdentifier(reviewHistoryEntryItem);
        if (reviewHistoryEntryIdentifier == null || reviewHistoryEntryCollectionIdentifiers.includes(reviewHistoryEntryIdentifier)) {
          return false;
        }
        reviewHistoryEntryCollectionIdentifiers.push(reviewHistoryEntryIdentifier);
        return true;
      });
      return [...reviewHistoryEntriesToAdd, ...reviewHistoryEntryCollection];
    }
    return reviewHistoryEntryCollection;
  }

  protected convertDateFromClient(reviewHistoryEntry: IReviewHistoryEntry): IReviewHistoryEntry {
    return Object.assign({}, reviewHistoryEntry, {
      entryDate: reviewHistoryEntry.entryDate?.isValid() ? reviewHistoryEntry.entryDate.toJSON() : undefined,
    });
  }

  protected convertDateFromServer(res: EntityResponseType): EntityResponseType {
    if (res.body) {
      res.body.entryDate = res.body.entryDate ? dayjs(res.body.entryDate) : undefined;
    }
    return res;
  }

  protected convertDateArrayFromServer(res: EntityArrayResponseType): EntityArrayResponseType {
    if (res.body) {
      res.body.forEach((reviewHistoryEntry: IReviewHistoryEntry) => {
        reviewHistoryEntry.entryDate = reviewHistoryEntry.entryDate ? dayjs(reviewHistoryEntry.entryDate) : undefined;
      });
    }
    return res;
  }
}
