import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { FormBuilder, Validators } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { Observable } from 'rxjs';
import { finalize, map } from 'rxjs/operators';

import dayjs from 'dayjs/esm';
import { DATE_TIME_FORMAT } from 'app/config/input.constants';

import { IReviewHistoryEntry, ReviewHistoryEntry } from '../review-history-entry.model';
import { ReviewHistoryEntryService } from '../service/review-history-entry.service';
import { IReview } from 'app/entities/review/review.model';
import { ReviewService } from 'app/entities/review/service/review.service';
import { IReviewRating } from 'app/entities/review-rating/review-rating.model';
import { ReviewRatingService } from 'app/entities/review-rating/service/review-rating.service';
import { IUser } from 'app/entities/user/user.model';
import { UserService } from 'app/entities/user/user.service';
import { ReviewStatus } from 'app/entities/enumerations/review-status.model';

@Component({
  selector: 'jhi-review-history-entry-update',
  templateUrl: './review-history-entry-update.component.html',
})
export class ReviewHistoryEntryUpdateComponent implements OnInit {
  isSaving = false;
  reviewStatusValues = Object.keys(ReviewStatus);

  reviewsSharedCollection: IReview[] = [];
  reviewRatingsSharedCollection: IReviewRating[] = [];
  usersSharedCollection: IUser[] = [];

  editForm = this.fb.group({
    id: [],
    message: [null, [Validators.maxLength(255)]],
    type: [null, [Validators.required]],
    entryDate: [null, [Validators.required]],
    review: [null, Validators.required],
    reviewRating: [],
    user: [],
  });

  constructor(
    protected reviewHistoryEntryService: ReviewHistoryEntryService,
    protected reviewService: ReviewService,
    protected reviewRatingService: ReviewRatingService,
    protected userService: UserService,
    protected activatedRoute: ActivatedRoute,
    protected fb: FormBuilder
  ) {}

  ngOnInit(): void {
    this.activatedRoute.data.subscribe(({ reviewHistoryEntry }) => {
      if (reviewHistoryEntry.id === undefined) {
        const today = dayjs().startOf('day');
        reviewHistoryEntry.entryDate = today;
      }

      this.updateForm(reviewHistoryEntry);

      this.loadRelationshipsOptions();
    });
  }

  previousState(): void {
    window.history.back();
  }

  save(): void {
    this.isSaving = true;
    const reviewHistoryEntry = this.createFromForm();
    if (reviewHistoryEntry.id !== undefined) {
      this.subscribeToSaveResponse(this.reviewHistoryEntryService.update(reviewHistoryEntry));
    } else {
      this.subscribeToSaveResponse(this.reviewHistoryEntryService.create(reviewHistoryEntry));
    }
  }

  trackReviewById(index: number, item: IReview): number {
    return item.id!;
  }

  trackReviewRatingById(index: number, item: IReviewRating): number {
    return item.id!;
  }

  trackUserById(index: number, item: IUser): number {
    return item.id!;
  }

  protected subscribeToSaveResponse(result: Observable<HttpResponse<IReviewHistoryEntry>>): void {
    result.pipe(finalize(() => this.onSaveFinalize())).subscribe({
      next: () => this.onSaveSuccess(),
      error: () => this.onSaveError(),
    });
  }

  protected onSaveSuccess(): void {
    this.previousState();
  }

  protected onSaveError(): void {
    // Api for inheritance.
  }

  protected onSaveFinalize(): void {
    this.isSaving = false;
  }

  protected updateForm(reviewHistoryEntry: IReviewHistoryEntry): void {
    this.editForm.patchValue({
      id: reviewHistoryEntry.id,
      message: reviewHistoryEntry.message,
      type: reviewHistoryEntry.type,
      entryDate: reviewHistoryEntry.entryDate ? reviewHistoryEntry.entryDate.format(DATE_TIME_FORMAT) : null,
      review: reviewHistoryEntry.review,
      reviewRating: reviewHistoryEntry.reviewRating,
      user: reviewHistoryEntry.user,
    });

    this.reviewsSharedCollection = this.reviewService.addReviewToCollectionIfMissing(
      this.reviewsSharedCollection,
      reviewHistoryEntry.review
    );
    this.reviewRatingsSharedCollection = this.reviewRatingService.addReviewRatingToCollectionIfMissing(
      this.reviewRatingsSharedCollection,
      reviewHistoryEntry.reviewRating
    );
    this.usersSharedCollection = this.userService.addUserToCollectionIfMissing(this.usersSharedCollection, reviewHistoryEntry.user);
  }

  protected loadRelationshipsOptions(): void {
    this.reviewService
      .query()
      .pipe(map((res: HttpResponse<IReview[]>) => res.body ?? []))
      .pipe(map((reviews: IReview[]) => this.reviewService.addReviewToCollectionIfMissing(reviews, this.editForm.get('review')!.value)))
      .subscribe((reviews: IReview[]) => (this.reviewsSharedCollection = reviews));

    this.reviewRatingService
      .query()
      .pipe(map((res: HttpResponse<IReviewRating[]>) => res.body ?? []))
      .pipe(
        map((reviewRatings: IReviewRating[]) =>
          this.reviewRatingService.addReviewRatingToCollectionIfMissing(reviewRatings, this.editForm.get('reviewRating')!.value)
        )
      )
      .subscribe((reviewRatings: IReviewRating[]) => (this.reviewRatingsSharedCollection = reviewRatings));

    this.userService
      .query()
      .pipe(map((res: HttpResponse<IUser[]>) => res.body ?? []))
      .pipe(map((users: IUser[]) => this.userService.addUserToCollectionIfMissing(users, this.editForm.get('user')!.value)))
      .subscribe((users: IUser[]) => (this.usersSharedCollection = users));
  }

  protected createFromForm(): IReviewHistoryEntry {
    return {
      ...new ReviewHistoryEntry(),
      id: this.editForm.get(['id'])!.value,
      message: this.editForm.get(['message'])!.value,
      type: this.editForm.get(['type'])!.value,
      entryDate: this.editForm.get(['entryDate'])!.value ? dayjs(this.editForm.get(['entryDate'])!.value, DATE_TIME_FORMAT) : undefined,
      review: this.editForm.get(['review'])!.value,
      reviewRating: this.editForm.get(['reviewRating'])!.value,
      user: this.editForm.get(['user'])!.value,
    };
  }
}
